using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;

namespace Sdgt.Widgets
{
	#region Event Delegates
	// This handler returns the sender and the ID of the mouse that generated the click
	// Note that this is not the best way to do it; it would be better to write a handler that generates the usual 'e...' structure
	public delegate void buttonClickHandler(object sender, int userID);
	#endregion

	public class SDGButton : Sdgt.SdgUserControl
	{
		#region Variables
		private System.ComponentModel.IContainer components = null;

		// An array that indicates if a mouse matching the array index is pressed (up to 30 mice).
		private bool [] pressed = new bool [30]; 
		private string caption;	// local storage of the Caption property
		private bool useLock;   // local storage of the UseLock property

		#endregion

		#region Constructor/Destructor
		public SDGButton()
		{
			// This call is required by the Windows Form Designer.
			InitializeComponent();

			// Set the value of the double-buffering style bits to true. Note this doesn't really work right.
			this.SetStyle(ControlStyles.DoubleBuffer |  ControlStyles.UserPaint, true);
			this.UpdateStyles();

			// Set the initial caption
			this.Caption = "SDG Button";  
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}
		#endregion

		#region Properties
		/// <summary>
		/// Property to get and set the text Caption that is displayed in the button.
		/// </summary>
		[Description("The text Caption that is displayed in the button."),Category("Appearance")] 
		public string Caption
		{
			set
			{
				caption = value;
				this.Invalidate ();
			}
			get
			{
				return caption;
			}
		}

		/// <summary>
		/// Property to get and set UseLock for how users are allowed to use the mouse.
		/// </summary>
		[Description("If true, only one person can use the button at a time. If false, both can use it.")]
		public bool UseLock
		{
			set
			{
				useLock = value;
			}
			get
			{
				return useLock;
			}
		}
		#endregion

		#region Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			// 
			// SDGButton
			// 
			this.Name = "SDGButton";
			this.Size = new System.Drawing.Size(152, 72);
			this.Resize += new System.EventHandler(this.SDGButton_Resize);
			this.SdgMouseDown += new Sdgt.SdgMouseEventHandler(this.SDGButton_SdgMouseDown);
			this.SdgMouseUp += new Sdgt.SdgMouseEventHandler(this.SDGButton_SdgMouseUp);
			this.Paint += new System.Windows.Forms.PaintEventHandler(this.SDGButton_Paint);

		}
		#endregion

		#region Event Declarations
		/// <summary>
		/// Event fired when the button is clicked for a particular user.
		/// </summary>
		[Description("Event fired when the Button is clicked.")]
		public event buttonClickHandler sdgClick;
		#endregion

		#region Standard Event Handlers

		// Force a paint whenver the button is resized so it will look correct
		private void SDGButton_Resize(object sender, System.EventArgs e)
		{
			this.Invalidate ();
		}

		// On a paint event, redraw the graphics for the entire control
		private void SDGButton_Paint(object sender, System.Windows.Forms.PaintEventArgs e)
		{
			drawControl();	
		}

		protected override void OnPaintBackground (System.Windows.Forms.PaintEventArgs e)
		{
			// Do nothing. This is to prevent flicker.
		}
		#endregion
	
		#region SDG Event Handlers
		
		// On a mouse down and depending on the UseLock setting,
		// record that that mouse is in a pressed state and force a repaint
		private void SDGButton_SdgMouseDown(object sender, Sdgt.SdgMouseEventArgs e)
		{
			//Ignore events if the button is not enabled.
			if (this.Enabled == false) return;

			if (e.Button == MouseButtons.Left) 
			{
				if (this.UseLock == false) //simultaneous use allowed
				{
					pressed[e.ID] = true;
					this.Invalidate ();
				} 
				else // Only allow this person to press it if no one else has.
				{
					bool none_pressed = true;
					foreach (bool b in pressed) 
					{
						if (b == true) none_pressed = false;
					}
					if (none_pressed) 
					{
						pressed[e.ID] = true;
						this.Invalidate ();
					}
				}
			}
		}

		// On a mouse up, record that that mouse is in an unpressed state and force a repaint
		// Also check to see if any all other mice are unpressed. If so, then generate a click event.
		private void SDGButton_SdgMouseUp(object sender, Sdgt.SdgMouseEventArgs e)
		{
			//Ignore events if the button is not enabled.
			if (this.Enabled == false) return;

			if (e.Button == MouseButtons.Left) 
			{
				if (this.UseLock == false || (this.UseLock == true && pressed[e.ID] == true)) 
				{
					pressed[e.ID] = false;
					this.Invalidate ();

					bool none_pressed = true;
					foreach (bool b in pressed) 
					{
						if (b == true) none_pressed = false;
					}
			
					if (none_pressed && sdgClick != null) sdgClick(this, e.ID);
				} 
			}
		}
		#endregion

		#region Drawing
		
		// Draw the button
		private void drawControl ()
		{
			// Depending on whether any mice are pressing the button, draw the appropriate button graphic. 
			if (!micePressed())
			{
				drawUnpressedButton ();
			} 
			else 
			{ 
				drawPressedButton ();
			}
		}


		// Draw the Button Graphics showing its 'unpressed' state
		private void drawUnpressedButton ()
		{
			Graphics gfx = this.CreateGraphics();
			Pen p;
			gfx.Clear (this.BackColor);
			
			drawImage ();

			Point ptl = new Point (0, 0);
			Point ptr = new Point (this.Width - 1, 0);
			Point pbl = new Point (0, this.Height - 1);
			Point pbr = new Point (this.Width - 1, this.Height - 1);
			
			//White lines on inside left side and top
			p = new Pen (SystemBrushes.ControlLightLight, 4);
			gfx.DrawLine (p, ptl, ptr);
			gfx.DrawLine (p, ptl, pbl);
			
			//Grey lines on inside right side and bottom
			p = new Pen (SystemBrushes.ControlDark, 3);
			gfx.DrawLine (p, pbl, pbr);
			gfx.DrawLine (p, pbr, ptr);

			//Dark lines surround entire button
			p = new Pen (SystemBrushes.ControlDarkDark, 1);
			gfx.DrawLine (p, ptl, ptr);
			gfx.DrawLine (p, ptl, pbl);
			gfx.DrawLine (p, pbl, pbr);
			gfx.DrawLine (p, pbr, ptr);

			drawCenteredCaption ( new Point ( (this.Width - 1) / 2, (this.Height - 1) / 2));
		}


		// Draw the Button Graphics showing its 'pressed' state
		private void drawPressedButton ()
		{
			Graphics gfx = this.CreateGraphics();
			Pen p;
			gfx.Clear (this.BackColor);
			
			drawImage ();

			Point ptl = new Point (0, 0);
			Point ptr = new Point (this.Width - 1, 0);
			Point pbl = new Point (0, this.Height - 1);
			Point pbr = new Point (this.Width - 1, this.Height - 1);
			
			//Dark lines on inside left side and top
			p = new Pen (SystemBrushes.ControlDarkDark, 4);
			gfx.DrawLine (p, ptl, ptr);
			gfx.DrawLine (p, ptl, pbl);
			
			//Dark lines surround entire button
			p = new Pen (SystemBrushes.ControlDarkDark, 1);
			gfx.DrawLine (p, ptl, ptr);
			gfx.DrawLine (p, ptl, pbl);
			gfx.DrawLine (p, pbl, pbr);
			gfx.DrawLine (p, pbr, ptr);

			// Shift the Caption over a bit to give it that 3d click feel
			drawCenteredCaption ( new Point ( (this.Width + 2)/2, (this.Height + 2)/2) );
		}


		// Drawing the Caption in the Button
		private void drawCenteredCaption (Point centerpt)
		{
			Graphics gfx = this.CreateGraphics();

			//Set the string format alignment to be centered 
			StringFormat strfmt = new StringFormat();
			strfmt.Alignment = StringAlignment.Center;
			strfmt.LineAlignment = StringAlignment.Center;
			
			// if the button is enabled, draw the font normally. Otherwise, draw it in a light color and embossed in white
			if (this.Enabled) 
			{
				Brush b = new SolidBrush (this.ForeColor); 
				gfx.DrawString (this.Caption, this.Font, b, centerpt, strfmt); 
			} 
			else 
			{
				gfx.DrawString (this.Caption, this.Font, Brushes.White, centerpt.X - 1, centerpt.Y - 1, strfmt); 
				gfx.DrawString (this.Caption, this.Font, Brushes.White, centerpt.X + 1, centerpt.Y + 1, strfmt); 
				gfx.DrawString (this.Caption, this.Font, SystemBrushes.ControlDark, centerpt, strfmt); 
			}
		}

		// If there is a background image, draw it on the button.
		private void drawImage ()
		{
			if (this.BackgroundImage != null)
			{
				Graphics gfx = this.CreateGraphics();
				gfx.DrawImage (this.BackgroundImage, 3, 3, this.Width - 6, this.Height - 6);
			}
		}
		#endregion

		#region Utility Routines
		// Return true if any mice are pressing the button
		private bool micePressed ()
		{
			bool isPressed = false;
			foreach (bool b in pressed)
			{
				if (true == b)
					isPressed = true;
			}
			return isPressed;
		}
		#endregion
	}
}

